/* 
 * ==========================================================
 * Artekit demo sketch for the AK-MCP4922 Dual 12-bit DAC
 * breakout board.
 * http://www.artekit.eu
 *
 * Connect the pins in the following way:
 *
 * Arduino UNO          AK-MCP4922
 * ==================================
 *   MOSI (11)             MOSI
 *   SCK (13)              SCK
 *   SS (10)               CS
 *   2                     SHDN
 *   3                     LDAC
 *   5V                    VCC
 *   GND                   GND
 *
 * This sketch outputs a 500Hz sine wave on the OUTA output.
 * 
 * ==========================================================
 */
 
#include <SPI.h>

/* Macros for moving pins */
#define SHDN_HIGH()	digitalWrite(2, HIGH)
#define SHDN_LOW()	digitalWrite(2, LOW)
#define LDAC_HIGH()	digitalWrite(3, HIGH)
#define LDAC_LOW()	digitalWrite(3, LOW)
#define CS_HIGH()	digitalWrite(10, HIGH)
#define CS_LOW()	digitalWrite(10, LOW)

/* Here follows variables to calculate the sine wave */
uint32_t sampleRate = 5000;
uint32_t frequency = 500;
double theta = 0;
float deltaTheta = 2*M_PI*( (float) frequency / (float) sampleRate);

/* This function generates a sample of the sine wave */
int16_t generateSample()
{
  int16_t ret = (int16_t) (0.8f * (65535/2) * sin(theta));
  theta += deltaTheta;
  if(theta>2*M_PI) theta = theta - 2*M_PI;

  return ret;
}

void setup() {

  /* Start SPI */
  SPI.begin();
  SPI.setDataMode(SPI_MODE0);

  /* Set these two pins as outputs (for SHDN and LDAC) */
  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  
  /* Reset CS */
  CS_HIGH();
  
  /* Turn on the DAC */
  SHDN_HIGH();
  
  /* Reset LDAC */
  LDAC_HIGH();
}

void loop() {
 
  uint32_t start, end;
  uint8_t data[2];
  uint16_t sample;
	
  start = micros();
  
  LDAC_HIGH();
  CS_LOW();
	
  sample = generateSample();
  
  /* Sign and 12-bit calculation */
  sample += 0x8000;
  sample = (sample >> 4) & 0xFFF;
	
  /* Set the DAC enable flag */
  sample |= 0x1000;
	
  data[0] = (sample >> 8) & 0xFF;
  data[1] = sample & 0xFF;
  
  /* Send through SPI */
  SPI.transfer(data[0]);
  SPI.transfer(data[1]);

  CS_HIGH();
  LDAC_LOW();
  
  end = micros();
  
  /* Here we wait to send the next sample.
   * Since we are sending 5000 samples per second:
   * 1000ms / 5000 samples = +-200uS per sample. 
   * Magic number 195 was obtained using a DSO
   * to get around 500Hz on OUTA.
   */
  delayMicroseconds(195 - (end - start));
}
